@@ -1,6 +1,6 @@
-'''
+"""
 Check that various monitors work correctly.
-'''
+"""
 
 
 from brian2 import *
@@ -17,10 +17,10 @@ class SpikeMonitorTest(FeatureTest):
     def run(self):
         N = 100
         tau = 10*ms
-        eqs = '''
+        eqs = """
         dv/dt = (I-v)/tau : 1
         I : 1
-        '''
+        """
         self.G = G = NeuronGroup(N, eqs, threshold='v>1', reset='v=0')
         G.I = linspace(0, 2, N)
         self.M = M = SpikeMonitor(G)
@@ -42,10 +42,10 @@ class StateMonitorTest(FeatureTest):
     def run(self):
         N = 10
         tau = 10*ms
-        eqs = '''
+        eqs = """
         dv/dt = (I-v)/tau : 1
         I : 1
-        '''
+        """
         self.G = G = NeuronGroup(N, eqs, threshold='v>1', reset='v=0.1')
         G.v = 0.1
         G.I = linspace(1.1, 2, N)
