@@ -1,12 +1,16 @@
-
 from collections import namedtuple
+import json
 import os
+import platform
+import socket
 
 import numpy as np
 import pytest
 
 from brian2 import prefs, clear_cache, _cache_dirs_and_extensions
+from brian2.codegen.cpp_prefs import compiler_supports_c99, get_compiler_and_args
 from brian2.codegen.optimisation import optimise_statements
+from brian2.codegen.runtime.cython_rt import CythonCodeObject
 from brian2.codegen.translation import (analyse_identifiers,
                                         get_identifiers_recursively,
                                         parse_statement,
@@ -20,6 +24,7 @@ from brian2.core.functions import Function, DEFAULT_FUNCTIONS, DEFAULT_CONSTANTS
 from brian2.devices.device import auto_target, device
 from brian2.units.fundamentalunits import Unit
 from brian2.units import second, ms
+from brian2.utils.logger import catch_logs
 
 FakeGroup = namedtuple('FakeGroup', ['variables'])
 
@@ -31,13 +36,13 @@ def test_auto_target():
 
 @pytest.mark.codegen_independent
 def test_analyse_identifiers():
-    '''
+    """
     Test that the analyse_identifiers function works on a simple clear example.
-    '''
-    code = '''
+    """
+    code = """
     a = b+c
     d = e+f
-    '''
+    """
     known = {'b': Variable(name='b'),
              'c': Variable(name='c'),
              'd': Variable(name='d'),
@@ -52,9 +57,9 @@ def test_analyse_identifiers():
 
 @pytest.mark.codegen_independent
 def test_get_identifiers_recursively():
-    '''
+    """
     Test finding identifiers including subexpressions.
-    '''
+    """
     variables = {'sub1': Subexpression(name='sub1',
                                        dtype=np.float32, expr='sub2 * z',
                                        owner=FakeGroup(variables={}),
@@ -96,19 +101,19 @@ def test_repeated_subexpressions():
     }
     # subexpression a (referring to z) is used twice, but can be reused the
     # second time (no change to z)
-    code = '''
+    code = """
     x = a
     y = a
-    '''
+    """
     scalar_stmts, vector_stmts = make_statements(code, variables, np.float32)
     assert len(scalar_stmts) == 0
     assert [stmt.var for stmt in vector_stmts] == ['a', 'x', 'y']
     assert vector_stmts[0].constant
 
-    code = '''
+    code = """
     x = a
     z *= 2
-    '''
+    """
     scalar_stmts, vector_stmts = make_statements(code, variables, np.float32)
     assert len(scalar_stmts) == 0
     assert [stmt.var for stmt in vector_stmts] == ['a', 'x', 'z']
@@ -119,11 +124,11 @@ def test_repeated_subexpressions():
 
     # a refers to z, therefore we have to redefine a after z changed, and a
     # cannot be constant
-    code = '''
+    code = """
     x = a
     z *= 2
     y = a
-    '''
+    """
     scalar_stmts, vector_stmts = make_statements(code, variables, np.float32)
     assert len(scalar_stmts) == 0
     assert [stmt.var for stmt in vector_stmts] == ['a', 'x', 'z', 'a', 'y']
@@ -132,16 +137,16 @@ def test_repeated_subexpressions():
 
 @pytest.mark.codegen_independent
 def test_nested_subexpressions():
-    '''
+    """
     This test checks that code translation works with nested subexpressions.
-    '''
-    code = '''
+    """
+    code = """
     x = a + b + c
     c = 1
     x = a + b + c
     d = 1
     x = a + b + c
-    '''
+    """
     variables = {
         'a': Subexpression(name='a', dtype=np.float32, owner=FakeGroup(variables={}), device=None,
                            expr='b*b+d'),
@@ -249,23 +254,23 @@ def test_apply_loop_invariant_optimisation_no_optimisation():
                  'rand': DEFAULT_FUNCTIONS['rand']
                  }
     statements = [
-        # This hould not be simplified to 0!
-        Statement('v1', '=', 'rand() - rand()', '', np.float),
-        Statement('v1', '=', '3*rand() - 3*rand()', '', np.float),
-        Statement('v1', '=', '3*rand() - ((1+2)*rand())', '', np.float),
+        # This should not be simplified to 0!
+        Statement('v1', '=', 'rand() - rand()', '', float),
+        Statement('v1', '=', '3*rand() - 3*rand()', '', float),
+        Statement('v1', '=', '3*rand() - ((1+2)*rand())', '', float),
         # This should not pull out rand()*N
-        Statement('v1', '=', 's1*rand()*N', '', np.float),
-        Statement('v1', '=', 's2*rand()*N', '', np.float),
+        Statement('v1', '=', 's1*rand()*N', '', float),
+        Statement('v1', '=', 's2*rand()*N', '', float),
         # This is not important mathematically, but it would change the numbers
         # that are generated
-        Statement('v1', '=', '0*rand()*N', '', np.float),
-        Statement('v1', '=', '0/rand()*N', '', np.float)
+        Statement('v1', '=', '0*rand()*N', '', float),
+        Statement('v1', '=', '0/rand()*N', '', float)
     ]
     scalar, vector = optimise_statements([], statements, variables)
     for vs in vector[:3]:
-        assert vs.expr.count('rand()') == 2, 'Expression should still contain two rand() calls, but got ' + str(vs)
+        assert vs.expr.count('rand()') == 2, f"Expression should still contain two rand() calls, but got {str(vs)}"
     for vs in vector[3:]:
-        assert vs.expr.count('rand()') == 1, 'Expression should still contain a rand() call, but got ' + str(vs)
+        assert vs.expr.count('rand()') == 1, f"Expression should still contain a rand() call, but got {str(vs)}"
 
 @pytest.mark.codegen_independent
 def test_apply_loop_invariant_optimisation_simplification():
@@ -276,26 +281,26 @@ def test_apply_loop_invariant_optimisation_simplification():
                  }
     statements = [
         # Should be simplified to 0.0
-        Statement('v1', '=', 'v1 - v1', '', np.float),
-        Statement('v1', '=', 'N*v1 - N*v1', '', np.float),
-        Statement('v1', '=', 'v1*N * 0', '', np.float),
-        Statement('v1', '=', 'v1 * 0', '', np.float),
-        Statement('v1', '=', 'v1 * 0.0', '', np.float),
-        Statement('v1', '=', '0.0 / (v1*N)', '', np.float),
+        Statement('v1', '=', 'v1 - v1', '', float),
+        Statement('v1', '=', 'N*v1 - N*v1', '', float),
+        Statement('v1', '=', 'v1*N * 0', '', float),
+        Statement('v1', '=', 'v1 * 0', '', float),
+        Statement('v1', '=', 'v1 * 0.0', '', float),
+        Statement('v1', '=', '0.0 / (v1*N)', '', float),
         # Should be simplified to 0
-        Statement('i1', '=', 'i1*N * 0', '', np.int),
-        Statement('i1', '=', '0 * i1', '', np.int),
-        Statement('i1', '=', '0 * i1*N', '', np.int),
-        Statement('i1', '=', 'i1 * 0', '', np.int),
+        Statement('i1', '=', 'i1*N * 0', '', int),
+        Statement('i1', '=', '0 * i1', '', int),
+        Statement('i1', '=', '0 * i1*N', '', int),
+        Statement('i1', '=', 'i1 * 0', '', int),
         # Should be simplified to v1*N
-        Statement('v2', '=', '0 + v1*N', '', np.float),
-        Statement('v2', '=', 'v1*N + 0.0', '', np.float),
-        Statement('v2', '=', 'v1*N - 0', '', np.float),
-        Statement('v2', '=', 'v1*N - 0.0', '', np.float),
-        Statement('v2', '=', '1 * v1*N', '', np.float),
-        Statement('v2', '=', '1.0 * v1*N', '', np.float),
-        Statement('v2', '=', 'v1*N / 1.0', '', np.float),
-        Statement('v2', '=', 'v1*N / 1', '', np.float),
+        Statement('v2', '=', '0 + v1*N', '', float),
+        Statement('v2', '=', 'v1*N + 0.0', '', float),
+        Statement('v2', '=', 'v1*N - 0', '', float),
+        Statement('v2', '=', 'v1*N - 0.0', '', float),
+        Statement('v2', '=', '1 * v1*N', '', float),
+        Statement('v2', '=', '1.0 * v1*N', '', float),
+        Statement('v2', '=', 'v1*N / 1.0', '', float),
+        Statement('v2', '=', 'v1*N / 1', '', float),
         # Should be simplified to i1
         Statement('i1', '=', 'i1*1', '', int),
         Statement('i1', '=', 'i1//1', '', int),
@@ -352,9 +357,9 @@ def test_apply_loop_invariant_optimisation_constant_evaluation():
                  'exp': DEFAULT_FUNCTIONS['exp']
                  }
     statements = [
-        Statement('v1', '=', 'v1 * (1 + 2 + 3)', '', np.float),
-        Statement('v1', '=', 'exp(N)*v1', '', np.float),
-        Statement('v1', '=', 'exp(0)*v1', '', np.float),
+        Statement('v1', '=', 'v1 * (1 + 2 + 3)', '', float),
+        Statement('v1', '=', 'exp(N)*v1', '', float),
+        Statement('v1', '=', 'exp(0)*v1', '', float),
     ]
     scalar, vector = optimise_statements([], statements, variables)
     # exp(N) should be pulled out of the vector statements, the rest should be
@@ -381,7 +386,7 @@ def test_automatic_augmented_assignments():
         'z': ArrayVariable('y', owner=None, size=10,
                            device=device),
         'b': ArrayVariable('b', owner=None, size=10,
-                           dtype=np.bool, device=device),
+                           dtype=bool, device=device),
         'clip': DEFAULT_FUNCTIONS['clip'],
         'inf': DEFAULT_CONSTANTS['inf']
     }
@@ -407,19 +412,20 @@ def test_automatic_augmented_assignments():
     for orig, rewritten in statements:
         scalar, vector = make_statements(orig, variables, np.float32)
         try:  # we augment the assertion error with the original statement
-            assert len(scalar) == 0, 'Did not expect any scalar statements but got ' + str(scalar)
-            assert len(vector) == 1, 'Did expect a single statement but got ' + str(vector)
+            assert len(scalar) == 0, f"Did not expect any scalar statements but got {str(scalar)}"
+            assert len(vector) == 1, f"Did expect a single statement but got {str(vector)}"
             statement = vector[0]
             expected_var, expected_op, expected_expr, _ = parse_statement(rewritten)
-            assert expected_var == statement.var, 'expected write to variable %s, not to %s' % (expected_var, statement.var)
-            assert expected_op == statement.op, 'expected operation %s, not %s' % (expected_op, statement.op)
+            assert expected_var == statement.var, f'expected write to variable {expected_var}, not to {statement.var}'
+            assert expected_op == statement.op, f'expected operation {expected_op}, not {statement.op}'
             # Compare the two expressions using sympy to allow for different order etc.
             sympy_expected = str_to_sympy(expected_expr)
             sympy_actual = str_to_sympy(statement.expr)
-            assert sympy_expected == sympy_actual, ('RHS expressions "%s" and "%s" are not identical' % (sympy_to_str(sympy_expected),
-                                                                                                         sympy_to_str(sympy_actual)))
+            assert sympy_expected == sympy_actual, (
+                f"RHS expressions '{sympy_to_str(sympy_expected)}' and '{sympy_to_str(sympy_actual)}' are not identical")
         except AssertionError as ex:
-            raise AssertionError('Transformation for statement "%s" gave an unexpected result: %s' % (orig, str(ex)))
+            raise AssertionError(
+                f"Transformation for statement '{orig}' gave an unexpected result: {ex}")
 
 
 def test_clear_cache():
@@ -440,6 +446,91 @@ def test_clear_cache():
 
         os.remove(fname)
 
+@pytest.mark.skipif(platform.system() == 'Windows',
+                    reason='CC and CXX variables are ignored on Windows.')
+def test_compiler_error():
+    # In particular on OSX with clang in a conda environment, compilation might fail.
+    # Switching to a system gcc might help in such cases. Make sure that the error
+    # message mentions that.
+    old_CC = os.environ.get('CC', None)
+    old_CXX = os.environ.get('CXX', None)
+    os.environ.update({'CC': 'non-existing-compiler',
+                       'CXX': 'non-existing-compiler++'})
+    try:
+        with catch_logs() as l:
+            assert not CythonCodeObject.is_available()
+        assert len(l) > 0  # There are additional warnings about compiler flags
+        last_warning = l[-1]
+        assert last_warning[1].endswith('.failed_compile_test')
+        assert 'CC' in last_warning[2] and 'CXX' in last_warning[2]
+
+    finally:
+        if old_CC:
+            os.environ['CC'] = old_CC
+        else:
+            del os.environ['CC']
+        if old_CXX:
+            os.environ['CXX'] = old_CXX
+        else:
+            del os.environ['CXX']
+
+
+def test_compiler_c99():
+    # On a user's computer, we do not know whether the compiler actually
+    # has C99 support, so we just check whether the test does not raise an
+    # error
+
+    # The compiler check previously created spurious '-.o' files (see #1348)
+    if os.path.exists('-.o'):
+        os.remove('-.o')
+    c99_support = compiler_supports_c99()
+    assert not os.path.exists('-.o')
+    # On our Azure test server we know that the compilers support C99
+    if os.environ.get('AGENT_OS', ''):
+        assert c99_support
+
+
+def test_cpp_flags_support():
+    from distutils.ccompiler import get_default_compiler
+    from brian2.codegen.cpp_prefs import _compiler_flag_compatibility
+    _compiler_flag_compatibility.clear()  # make sure cache is empty
+    compiler = get_default_compiler()
+    if compiler == 'msvc':
+        pytest.skip('No flag support check for msvc')
+    old_prefs = prefs['codegen.cpp.extra_compile_args']
+
+    # Should always be supported
+    prefs['codegen.cpp.extra_compile_args'] = ['-w']
+    _, compile_args = get_compiler_and_args()
+    assert compile_args == prefs['codegen.cpp.extra_compile_args']
+
+    # Should never be supported and raise a warning
+    prefs['codegen.cpp.extra_compile_args'] = ['-invalidxyz']
+    with catch_logs() as l:
+        _, compile_args = get_compiler_and_args()
+    assert len(l) == 1 and l[0][0] == 'WARNING'
+    assert compile_args == []
+
+    prefs['codegen.cpp.extra_compile_args'] = old_prefs
+
+
+@pytest.mark.skipif(platform.system() != 'Windows',
+                    reason='MSVC flags are only relevant on Windows')
+@pytest.mark.skipif(prefs['codegen.target'] == 'numpy',
+                    reason='Test only relevant for compiled code')
+def test_msvc_flags():
+    # Very basic test that flags are stored to disk
+    import brian2.codegen.cpp_prefs as cpp_prefs
+    user_dir = os.path.join(os.path.expanduser('~'), '.brian')
+    flag_file = os.path.join(user_dir, 'cpu_flags.txt')
+    assert len(cpp_prefs.msvc_arch_flag)
+    assert os.path.exists(flag_file)
+    with open(flag_file, 'r', encoding='utf-8') as f:
+        previously_stored_flags = json.load(f)
+    hostname = socket.gethostname()
+    assert hostname in previously_stored_flags
+    assert len(previously_stored_flags[hostname])
+
 
 if __name__ == '__main__':
     test_auto_target()
@@ -456,4 +547,4 @@ if __name__ == '__main__':
     test_apply_loop_invariant_optimisation_constant_evaluation()
     test_automatic_augmented_assignments()
     test_clear_cache()
-
+    test_msvc_flags()
