@@ -9,6 +9,7 @@ Preferences
 import atexit
 import logging
 import logging.handlers
+from logging.handlers import RotatingFileHandler
 import os
 import shutil
 import sys
@@ -67,7 +68,7 @@ if 'logging' not in prefs.pref_register:
             """,
             ),
         file_log_level=BrianPreference(
-            default='DIAGNOSTIC',
+            default='DEBUG',
             docs="""
             What log level to use for the log written to the log file.
             
@@ -324,6 +325,9 @@ class BrianLogger(object):
     #: deleted after the run if no exception occurred), if any
     tmp_script = None
 
+    #: The pid of the process that initialized the logger – used to switch off file logging in
+    #: multiprocessing contexts
+    _pid = None
     def __init__(self, name):
         self.name = name
 
@@ -346,6 +350,12 @@ class BrianLogger(object):
         if name_suffix:
             name += f".{name_suffix}"
         
+        # Switch off file logging when using multiprocessing
+        if BrianLogger.tmp_log is not None and BrianLogger._pid != os.getpid():
+            BrianLogger.tmp_log = None
+            logging.getLogger('brian2').removeHandler(BrianLogger.file_handler)
+            BrianLogger.file_handler = None
+
         if once:
             # Check whether this exact message has already been displayed 
             log_tuple = (name, log_level, msg)
@@ -548,20 +558,25 @@ class BrianLogger(object):
         if prefs['logging.file_log']:
             try:
                 # Temporary filename used for logging
-                BrianLogger.tmp_log = tempfile.NamedTemporaryFile(prefix='brian_debug_',
-                                                                  suffix='.log',
-                                                                  delete=False)
-                BrianLogger.tmp_log = BrianLogger.tmp_log.name
+                with tempfile.NamedTemporaryFile(prefix='brian_debug_',
+                                                 suffix='.log',
+                                                 delete=False) as tmp_f:
+                    BrianLogger.tmp_log = tmp_f.name
+                # Remove any previously existing file handler
+                if BrianLogger.file_handler is not None:
+                    BrianLogger.file_handler.close()
+                    logger.removeHandler(BrianLogger.file_handler)
                 # Rotate log file after prefs['logging.file_log_max_size'] bytes and keep one copy
-                BrianLogger.file_handler = logging.handlers.RotatingFileHandler(BrianLogger.tmp_log,
-                                                                                mode='a',
-                                                                                maxBytes=prefs['logging.file_log_max_size'],
-                                                                                backupCount=1)
+                BrianLogger.file_handler = RotatingFileHandler(BrianLogger.tmp_log,
+                                                               mode='a',
+                                                               maxBytes=prefs['logging.file_log_max_size'],
+                                                               backupCount=1)
                 BrianLogger.file_handler.setLevel(
                     LOG_LEVELS[prefs['logging.file_log_level'].upper()])
                 BrianLogger.file_handler.setFormatter(logging.Formatter(
                     '%(asctime)s %(levelname)-10s %(name)s: %(message)s'))
                 logger.addHandler(BrianLogger.file_handler)
+                BrianLogger._pid = os.getpid()
             except IOError as ex:
                 warn(f'Could not create log file: {ex}')
 
@@ -592,6 +607,9 @@ class BrianLogger(object):
                     warn(
                         f'Could not copy script file to temp directory: {ex}')
 
+        if BrianLogger.console_handler is not None:
+            logger.removeHandler(BrianLogger.console_handler)
+
         # create console handler with a higher log level
         BrianLogger.console_handler = logging.StreamHandler()
         BrianLogger.console_handler.setLevel(LOG_LEVELS[prefs['logging.console_log_level']])
@@ -614,10 +632,10 @@ class BrianLogger(object):
             warn_logger.addHandler(BrianLogger.file_handler)
 
         # Put some standard info into the log file
-        logger.log(DIAGNOSTIC,
+        logger.log(logging.DEBUG,
                    f'Logging to file: {BrianLogger.tmp_log}, copy of main script saved as: {BrianLogger.tmp_script}')
-        logger.log(DIAGNOSTIC, f'Python interpreter: {sys.executable}')
-        logger.log(DIAGNOSTIC, f'Platform: {sys.platform}')
+        logger.log(logging.DEBUG, f'Python interpreter: {sys.executable}')
+        logger.log(logging.DEBUG, f'Platform: {sys.platform}')
         version_infos = {'brian': brian2.__version__,
                          'numpy': numpy.__version__,
                          'scipy': scipy.__version__ if scipy else 'not installed',
@@ -625,7 +643,7 @@ class BrianLogger(object):
                          'python': sys.version,
                          }
         for _name, _version in version_infos.items():
-            logger.log(DIAGNOSTIC,
+            logger.log(logging.DEBUG,
                        f'{_name} version is: {str(_version)}')
         # Handle uncaught exceptions
         sys.excepthook = brian_excepthook
